################
# MyScreen Viewer Applet
# By Danny Allen
################

import karamba
import urllib2
import os.path
from os.path import expanduser
import commands

#this is called when you widget is initialized
def initWidget(widget):
	global version,appletType,host,site,homeDir,appletDir,currentFile,timestampFileName
	global timestampPath,logo,imgFrame,img,imageUrl,nickButtonImg,nickStatusImg,nickButton
	global nickStatusIndicator,watchNick,timestampEngine,menu1,idExpanded,softForceReload
	global forceReload,getFreshImg,serviceStatus

	forceReload = 0
	softForceReload = 0
	getFreshImg = 1

	# add "config menu" keys
	karamba.addMenuConfigOption(widget, "autoReload", "Auto Reload Image?")
	karamba.addMenuConfigOption(widget, "appletClick", "Applet Clickable?")
	karamba.addMenuConfigOption(widget, "checkUpdates", "Check For Applet Updates...")
	readAppletClick = karamba.readMenuConfigOption(widget, "appletClick")
	karamba.setMenuConfigOption(widget, "checkUpdates", 0)

	# define variables
	version = 1
	appletType = "default"
	host = "http://myscreen.vidsolbach.de"
	site = "http://myscreen.org"
	homeDir = expanduser('~')
	appletDir = karamba.getThemePath(widget)
	currentFile = homeDir + "/.superkaramba/" + "current.png"
	timestampFileName = "timestamp.txt"
	timestampPath = homeDir + "/.superkaramba/" + timestampFileName

	# draw background image and loading pane
	bg = karamba.createBackgroundImage(widget,0,0,"img/bg.png")
	imgFrame = karamba.createImage(widget,7,32,"img/loading_pane.png")

	# draw logo image (create button?)
	logo = karamba.createImage(widget,8,0,"img/logo.png")
	# make logo clickable depending on user setting
	if (readAppletClick == 1):
		karamba.attachClickArea(widget, logo)

	karamba.redrawWidget(widget)

	# get service status
	try:
		print "Checking service status..."
		statusUrl = "%s/applet/servicestatus.txt" %(site)
		statusGet = urllib2.urlopen(statusUrl).read()
		statusFile = homeDir + "/.superkaramba/servicestatus.txt"
		saveStatusFile = open(statusFile, 'w')
		print >> saveStatusFile, statusGet
		saveStatusFile.close()
		# check service status
		serviceStatus = commands.getoutput("sh %s/scripts/servicestatus.sh" %(appletDir))
	except:
		serviceStatus = "0"

	# which nick is being watched by applet?
	watchNick = karamba.readConfigEntry(widget, "showNick")
	if (watchNick == 0) or (watchNick == None):
		# config key doesn't exist - first run
		print "Config key doesn't exist - creating..."
		watchNick = 1
		karamba.writeConfigEntry(widget, "showNick", "%s" %(watchNick))

	print "-" *20

	# get user_translation file
	try:
		userFile = "user_translation.txt"
		userFileUrl = "%s/%s" %(site,userFile)
		userListNicks = urllib2.urlopen(userFileUrl).read()
		userListUserFile = homeDir + "/.superkaramba/" + userFile
		saveUserListUserFile = open(userListUserFile, 'w')
		print >> saveUserListUserFile, userListNicks
		print "Downloaded MyScreen user list."
		saveUserListUserFile.close()
	except:
		print "Could not download MyScreen user list!"
		print "Are you connected to the internet?"
		print "If you are, check the status at go to http://myscreen.org/applet"

	# get number of lines in userfile
	linesUserList = 0
	userFileOpen = open(userListUserFile, 'r')
	for line in userFileOpen:
		linesUserList += 1
	userFileOpen.close()

	# read numUsers config key, so that we can compare with latest number of users
	readNumUsers = karamba.readConfigEntry(widget, "numUsers")
	if (readNumUsers == 0) or (readNumUsers == None):
		# config key doesn't exist - first run
		karamba.writeConfigEntry(widget, "numUsers", "0")
		print "Created user config key."

	if (linesUserList > readNumUsers):
		# there are new users that the applet doesn't know about, so recreate config entries
		print "There are new users on MyScreen that I don't know about!..."
		bMark = 1

		while (bMark <= linesUserList):
			# write nicks to config keys
			listLine = commands.getoutput("sh %s/scripts/listparser.sh %s" %(appletDir,bMark))
			print "User", bMark, "is", listLine
			karamba.writeConfigEntry(widget, "nick%s" %(bMark), "%s" %(listLine))
			# write nums to config keys
			karamba.writeConfigEntry(widget, "num%s" %(bMark), "%s" %(bMark))
			# check if we have image assets for user
			buttonExists = os.path.isfile("%s/img/%s.png" %(appletDir,bMark))
			if (buttonExists == False):
				try:
					buttonUrl = "%s/applet/img/default/%s.png" %(site,bMark)
					buttonFile = appletDir + "/img/" + "%s.png" %(bMark)
					buttonImg = urllib2.urlopen(buttonUrl).read()
					saveButtonImg = open(buttonFile, 'w')
					print >> saveButtonImg, buttonImg
					saveButtonImg.close()
					print "Downloaded image assets for %s." %(listLine)
				except:
					print "Could not download image assets for %s!" %(listLine)
					print "Are you sure you are connected to the internet?"
					print "If you are, check the status at go to http://myscreen.org/applet"

			bMark += 1
		# create config key, so that we only refresh user list on new users
		karamba.writeConfigEntry(widget, "numUsers", "%s" %(linesUserList))
	else:
		print "I know about all MyScreen users."

	print "-" *20

	karamba.redrawWidget(widget)

	# get status of user
	nom = karamba.readConfigEntry(widget, "nick%s" %(watchNick))
	try:
		timestampUrl = "%s/%s/%s" %(host,watchNick,timestampFileName)
		timestamp = urllib2.urlopen(timestampUrl).read()
		timestampFile = open(timestampPath, 'w')
		print >> timestampFile, timestamp
		timestampFile.close()
	
		# run timestamp comparison engine
		timestampEng = commands.getoutput("sh %s/scripts/timestamp.sh" %(appletDir))
	except:
		timestampEng = "0"
		print "Could not check status of %s!" &(nom)
		print "Are you sure you are connected to the internet?"
		print "If you are, check the status at go to http://myscreen.org/applet"

	if (timestampEng == "0"):
		print "%s is offline." %(nom)
		nickStatusImg = "img/off.png"
	elif (timestampEng == "1"):
		print "%s is online!" %(nom)
		nickStatusImg = "img/live.png"

	# define images to use depending on selected user
	nickButtonImg = "img/%s.png" %(watchNick)
	if (serviceStatus == "0"):
		print "Service is DOWN!"
		currentFileToUse = "img/fallback.png"
	elif (serviceStatus == "1"):
		# refetch current image
		try:
			imageUrl = "%s/%s/current.png" %(host,watchNick)
			currentImg = urllib2.urlopen(imageUrl).read()
			saveImg = open(currentFile, 'w')
			print >> saveImg, currentImg
			saveImg.close()
			currentFileToUse = currentFile
		except:
			currentFileToUse = "img/fallback.png"
			print "Could not get current image!"
			print "Are you sure you are connected to the internet?"
			print "If you are, check the status at go to http://myscreen.org/applet"

	# create current image
	img = karamba.createImage(widget,11,36,currentFileToUse)
	currentImgWidth = karamba.getImageWidth(widget, img)
	currentImgHeight = karamba.getImageHeight(widget, img)
	if (currentImgWidth != "314") or (currentImgHeight != "236"):
		# current img not the correct size, so lets resize it
		karamba.resizeImage(widget, img, 314, 236)
	# make current image clickable depending on user setting
	if (readAppletClick == 1):
		karamba.attachClickArea(widget, img)

	print "-" *20

	# create nick selection button
	nickButton = karamba.createImage(widget,0,278,nickButtonImg)
	karamba.attachClickArea(widget, nickButton)

	# create nick status indicator
	nickStatusIndicator = karamba.createImage(widget,226,278,nickStatusImg)
	karamba.attachClickArea(widget, nickStatusIndicator)

	# redraw widget
	karamba.redrawWidget(widget)

	# create nick selection menu
	menu1 = karamba.createMenu(widget)
	mMark = 1
	while (mMark <= linesUserList):
		nom = karamba.readConfigEntry(widget, "nick%s" %(mMark))
		idExpanded = karamba.addMenuItem(widget, menu1, "%s" %(nom), "%s/img/smalluser.png" %(appletDir))

		print "Created menu entry for %s, idnum: %s" %(nom,idExpanded)
		mMark += 1
	mMark = 1


#this is called everytime your widget is updated
#the update inverval is specified in the .theme file
def widgetUpdated(widget):
	global softForceReload,forceReload,nickStatusIndicator,appletDir,logo,img,imageUrl,nickButton
	global nickButtonImg,host,watchNick,nickStatusImg,serviceStatus,getFreshImg,currentFileToUse

	if (serviceStatus == "0"):
		# if service is down, recheck service status
		try:
			print "Checking service status..."
			statusUrl = "%s/applet/servicestatus.txt" %(site)
			statusGet = urllib2.urlopen(statusUrl).read()
			statusFile = homeDir + "/.superkaramba/servicestatus.txt"
			saveStatusFile = open(statusFile, 'w')
			print >> saveStatusFile, statusGet
			saveStatusFile.close()
			# check service status
			serviceStatus = commands.getoutput("sh %s/scripts/servicestatus.sh" %(appletDir))
		except:
			serviceStatus = "0"
			print "Could not check service status!"
			print ""
			print "There are 2 possibilities:"
			print "* You are not connected to the internet."
			print "* The MyScreen service is deceased!"

	# redraw widget to remove lingering menu artifacts in a soft forced reload - then reset variable
	if (softForceReload == 1):
		karamba.redrawWidget(widget)
		softForceReload = 0

	readAutoReload = karamba.readMenuConfigOption(widget, "autoReload")
	if (readAutoReload == 1) or (forceReload == 1):

		# read "config menu" keys
		readAppletClick = karamba.readMenuConfigOption(widget, "appletClick")

		# only redraw if user forced reload (for user satisfaction :)
		# - otherwise, refresh will interrupt regular background display)
		if (forceReload == 1):
			# delete nick status indicator, draw loading image, then redraw widget to show it
			karamba.deleteImage(widget,nickStatusIndicator)
			nickStatusIndicator = karamba.createImage(widget,226,278,"%s/img/connect.png" %(appletDir))
			karamba.redrawWidget(widget)

		# define image to use depending on selected user
		watchNick = karamba.readConfigEntry(widget, "showNick")
		nickButtonImg = "img/%s.png" %(watchNick)

		# delete then recreate create nick selection button
		karamba.deleteImage(widget,nickButton)
		nickButton = karamba.createImage(widget,0,278,nickButtonImg)
		karamba.attachClickArea(widget, nickButton)

		# delete current image (recreate shortly)
		karamba.deleteImage(widget,img)

		# only redraw if user forced reload (for user satisfaction :)
		# - otherwise, refresh will interrupt regular background display)
		if (forceReload == 1):
			karamba.redrawWidget(widget)

		# delete then redraw logo image (create button?)
		karamba.deleteImage(widget,logo)
		logo = karamba.createImage(widget,8,0,"img/logo.png")
		# make logo clickable depending on user setting
		if (readAppletClick == 1):
			karamba.attachClickArea(widget, logo)

		# define currentimg to use depending service status
		if (serviceStatus == "0"):
			print "Service is DOWN!"
			currentFileToUse = "img/fallback.png"
		elif (serviceStatus == "1"):
			# refetch current image if user is online
			if (getFreshImg == 1):
				print "Getting fresh user image..."
				try:
					imageUrl = "%s/%s/current.png" %(host,watchNick)
					currentImg = urllib2.urlopen(imageUrl).read()
					saveImg = open(currentFile, 'w')
					print >> saveImg, currentImg
					saveImg.close()
					currentFileToUse = currentFile
				except:
					currentFileToUse = "img/fallback.png"
					print "Are you sure you are connected to the internet?"
					print "If you are, check the status at go to http://myscreen.org/applet"

		# get status of user
		try:
			timestampUrl = "%s/%s/%s" %(host,watchNick,timestampFileName)
			timestamp = urllib2.urlopen(timestampUrl).read()
			timestampFile = open(timestampPath, 'w')
			print >> timestampFile, timestamp
			timestampFile.close()
	
			# run timestamp comparison engine
			timestampEng = commands.getoutput("sh %s/scripts/timestamp.sh" %(appletDir))
			nom = karamba.readConfigEntry(widget, "nick%s" %(watchNick))
		except:
			timestampEng = "0"
			print "Could not check status of %s!" &(nom)
			print "Are you sure you are connected to the internet?"
			print "If you are, check the status at go to http://myscreen.org/applet"

		# check if user is offline - if so, do not get fresh images in future
		if (timestampEng == "0"):
			print "%s is offline." %(nom)
			if (getFreshImg == 1):
				getFreshImg = 0
			nickStatusImg = "img/off.png"
		elif (timestampEng == "1"):
			print "%s is online!" %(nom)
			getFreshImg = 1
			nickStatusImg = "img/live.png"

		# recreate current image
		img = karamba.createImage(widget,11,36,currentFileToUse)
		# check to see if image is correct size
		currentImgWidth = karamba.getImageWidth(widget, img)
		currentImgHeight = karamba.getImageHeight(widget, img)
		if (currentImgWidth != "314") or (currentImgHeight != "236"):
			# current img not the correct size, so lets resize it
			karamba.resizeImage(widget, img, 314, 236)
		# make current image clickable depending on user setting
		if (readAppletClick == 1):
			karamba.attachClickArea(widget, img)

		print "-" *20

		# delete then recreate nick status indicator
		karamba.deleteImage(widget,nickStatusIndicator)
		nickStatusIndicator = karamba.createImage(widget,226,278,nickStatusImg)
		karamba.attachClickArea(widget, nickStatusIndicator)

		# check if this was a forced reload, if so, reset variable
		if (forceReload == 1):
			print "Forced Reload..."
			forceReload = 0

		print "Reloaded current image."

	# redraw widget
	karamba.redrawWidget(widget)


#This gets called when a meter (image, text, etc) is clicked.
# NOTE you must use attachClickArea() to make a meter
# clickable.
#  widget = reference to your theme
#  meter = the meter clicked
#  button = the button clicked (see widgetClicked for button numbers)
def meterClicked(widget, meter, button):
	global menu1,watchNick,appletDir,forceReload,softForceReload

	# if logo button is clicked, go to user page on website
	if(meter == logo):
		if(button == 1):
			print "-" *20
			print "Clicked Logo Button"
			print "-" *20
			userName = karamba.readConfigEntry(widget, "nick%s" %(watchNick))
			commands.getoutput("sh %s/scripts/userpagelaunch.sh %s" %(appletDir,userName))

	# if main image is clicked, refresh image
	if(meter == img):
		if(button == 1):
			print "-" *20
			print "Clicked Main Image"
			print "-" *20
			forceReload = 1
			widgetUpdated(widget)

	# if nick selection button is clicked, draw nick menu
	if(meter == nickButton):
		if(button == 1):
			print "-" *20
			print "Clicked User Menu"
			print "-" *20
			# position menu based on number of users to display, for perfect placement :)
			readNumUsers = karamba.readConfigEntry(widget, "numUsers")
			userMultiply = readNumUsers * 22
			menuPlacement = 275 - userMultiply
			karamba.popupMenu(widget, menu1, 8, menuPlacement)

	# if nick status iindcator is clicked, refresh image
	if(meter == nickStatusIndicator):
		if(button == 1):
			print "-" *20
			print "Clicked User Status Indicator"
			print "-" *20
			forceReload = 1
			widgetUpdated(widget)


#This gets called when an item is clicked in a popup menu you have created.
#  menu = a reference to the menu
#  id = the number of the item that was clicked.
def menuItemClicked(widget, menu, id):
	global menu1,idExpanded,idExpandedStore,imageUrl,img,nickButton,nickStatusIndicator,users
	global softForceReload,forceReload,getFreshImg

	users = karamba.readConfigEntry(widget, "numUsers")
	idExpandedStore = idExpanded

	if (menu == menu1):
		hMark = 1
		while (hMark <= users):
			if (id == idExpanded):
				# because we are checking backwards down idExpanded, flip no, then add 1
				# to correct hMark subversion
				flippedUserNo = users - hMark + 1
				karamba.writeConfigEntry(widget, "showNick", "%s" %(flippedUserNo))
				print "Selected user number", flippedUserNo

			idExpanded += 1
			hMark += 1
	# restore value for next menu operation
	idExpanded = idExpandedStore

	# changed user, so we must get fresh image regardless
	getFreshImg = 1
	forceReload = 1
	widgetUpdated(widget)


#This gets called when an item is clicked in the theme CONFIGURATION menu,
#not the popup menus that you create.
#  key = the reference to the configuration key that was changed
#  value = the new value (true or false) that was selected
def menuOptionChanged(widget, key, value):
	global forceReload,softForceReload

	karamba.redrawWidget(widget)

	if (key == "autoReload"):
		readAutoReload = karamba.readMenuConfigOption(widget, "autoReload")
		softForceReload = 1

	if (key == "appletClick"):
		readAppletClick = karamba.readMenuConfigOption(widget, "appletClick")
		forceReload = 1

	if (key == "checkUpdates"):
		global nickStatusIndicator,imgFrame,img,homeDir,appletDir,appletType,version

		readCheckUpdates = karamba.readMenuConfigOption(widget, "checkUpdates")

		# delete nick status indicator, draw loading image, then redraw widget to show it
		karamba.deleteImage(widget,nickStatusIndicator)
		nickStatusIndicator = karamba.createImage(widget,226,278,"%s/img/connect.png" %(appletDir))
		# delete image frame, replace with update pane, then delete current image and redraw
		karamba.deleteImage(widget,imgFrame)
		imgFrame = karamba.createImage(widget,7,32,"img/updatecheck_pane.png")
		karamba.deleteImage(widget,img)
		karamba.redrawWidget(widget)

		# get number of latest version
		try:
			print "Checking for latest version at", currentVersionUrl
			currentVersionFileName = "applet_version.txt"
			currentVersionPath = homeDir + "/.superkaramba/" + currentVersionFileName
			currentVersionUrl = "%s/applet/applet_version.txt" %(site)
			currentVersion = urllib2.urlopen(currentVersionUrl).read()
			currentVersionFile = open(currentVersionPath, 'w')
			print >> currentVersionFile, currentVersion
			currentVersionFile.close()
			
			# run update manager
			updateCheck = commands.getoutput("sh %s/scripts/update.sh %s %s %s" %(appletDir,version,appletDir,appletType))
		except:
			updatecheck = "5"

		if (updateCheck == "0"):
			# new version of applet available
			print "-" * 20
			print "Update downloaded and installed successfully."
			print "-" * 20
		elif (updateCheck == "1"):
			# update install failed
			print "-" * 20
			print "There was an error installing the applet update!"
			print "The update package is located at %s"
			print "if you want to attempt to install it manually" %(appletDir)
			print "-" * 20
		elif (updateCheck == "2"):
			print "-" * 20
			print "There was an error downloading the applet update!"
			print "-" * 20
		elif (updateCheck == "3"):
			print "-" * 20
			print "You have declined the available applet update."
			print "-" * 20
		elif (updateCheck == "4"):
			# already running current version of applet
			print "-" * 20
			print "You already have the latest version (R%s) of the MyScreen applet." %(version)
			print "-" * 20
		elif (updateCheck == "5"):
			# could not connect to server to check for updates
			print "-" * 20
			print "Could not check for updates!"
			print "Are you sure you are connected to the internet?"
			print "If you are, check the status at go to http://myscreen.org/applet"
			print "-" * 20

		# change back to loading pane, create a placeholder img so widgetupdate
		# has something to manipulate
		karamba.deleteImage(widget,imgFrame)
		imgFrame = karamba.createImage(widget,7,32,"img/loading_pane.png")
		img = karamba.createImage(widget,11,36,"img/updateblank.png")

		karamba.setMenuConfigOption(widget, "checkUpdates", 0)

		forceReload = 1
	widgetUpdated(widget)