#/bin/bash
# Update Manager
# By Danny Allen 2005

# set variables, some passed from applet
current_version=$1
applet_directory=$2
applet_type=$3
latest_version=`< ~/.superkaramba/applet_version.txt`

cd $applet_directory

if [ $latest_version -le $current_version ]; then
	# already running current version
	echo "4"
	exit 0;
fi

# ask if user wants to download and install new version
kdialog --title "Applet Update Available!" --yesno "There is an update available (R$latest_version) for the MyScreen applet - would you like to download and install it now?"
if [ $? = 0 ]; then
	# check if update package already exists locally - if so, remove it
	if [ -e update_$latest_version.tar.gz ]; then
		rm update_$latest_version.tar.gz
	fi
	# download the update package
	wget -q http://myscreen.vidsolbach.de/applet/update_$applet_type-$current_version-$latest_version.tar.gz
	# check if package downloaded ok
	if [ -e update_$applet_type-$current_version-$latest_version.tar.gz ]; then
		# extract update contents
		if tar --same-permissions --overwrite -zxf update_$applet_type-$current_version-$latest_version.tar.gz
		then
			# remove update package, then report success
			rm update_$applet_type-$current_version-$latest_version.tar.gz
			echo "0"
			exit 0;
		else
			# tell applet that we couldn't install update contents
			echo "1"
			exit 1;
		fi
		exit 0;
	else
		# error in downloading update
		echo "2"
		exit 1;
	fi
elif [ $? = 1 ]; then
	# user has declined update
	echo "3"
	exit 0;
fi